/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : SetCoverProblem.h                                        */
/*                                                                       */
/*       Date : March 4, 1998                                            */
/* Updated at : September 12, 1998                                       */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#ifndef _SETCOVERPROBLEM_H
#define _SETCOVERPROBLEM_H

#include <iostream.h>

#include <tSet.h>
#include <deffunc.h>

#include "SetCover.h"
#include "Column.h"

class SetCoverProblem
{ 
  public:

     SetCoverProblem(ulong,ulong,ulong,ulong,double);

     SetCoverProblem(const SetCoverProblem&);
     ~SetCoverProblem();
     void UpdateRow(ulong,ulong,ulong,ulong,int *);

     void addColumn(Column* c) { myColumns.addElement(c); };

     void buildLookUpTable();
     void printLookUpTable(ostream& os);

     void printRowInfo() const {printRowInfo(cout);};
     void printRowInfo(ostream&) const;


     unsigned int getNumOfCols();
     unsigned int getNumOfRows();
     Column   *getColumnWithIndx(int);

     getIndexOfSignature(char*);

     getIndexOfParSignature(char*);

     getIndexOfParSignature(int,char*);

     getIndexOfParSignature(int,char*,int);

     void getColumnSetCoveringRow(int indx, tSet<int>* ret);

     void sort();

     void findMinimums();
     cost_t getMinimumCostForRowIndex(unsigned int) const;
     float  getAverageNumOfColumns();
     float  getAverageNumOfRows();
     float  getDensity();

     friend ostream &operator<<(ostream&,const SetCoverProblem&);

  private:

          ulong NumOfRows;
          ulong NumOfColumns;
          ulong NumberOfNonZeros;
          ulong TheBestIP;
         double TheBestLP;
  tSet<Column*> myColumns;
         cost_t *cost;
   unsigned int *smallest;
      tSet<int> *LookUpTable;

      void sortAccordingToCost(int i);

/**
    LookUpTable[k] keeps the set of columns which cover flight k, in
    increasing order according to the cost value.

    myColumns : keeps the available columns.

    NumOfRows : denotes the number of rows that the solution is supposed
		to cover.

    NumOfColumns : denotes the number of available columns.

*/

}; // end of SetCoverProblem

#endif
